# -*- coding: utf-8 -*-
import copy
import urllib
import json
from django.conf import settings
from django.db import transaction
from django.template import loader
from djangoplus.ui.components.forms.fields import *
from djangoplus.ui.components.forms import widgets
from django import forms as django_forms
from django.forms.forms import BoundField
from djangoplus.templatetags import mobile
from django.utils.html import conditional_escape
from djangoplus.utils.metadata import get_metadata, iterable
from django.db.models.fields import NOT_PROVIDED
ValidationError = django_forms.ValidationError
DEFAULT_FORM_TITLE = 'Formulário'
DEFAULT_SUBMIT_LABEL = 'Enviar'


class Form(django_forms.Form):

    fieldsets = None

    def __init__(self, request, *args, **kwargs):
        metaclass = hasattr(self.__class__, 'Meta') and self.__class__.Meta or None
        self.request = request
        self.method = kwargs.pop('method', None) or metaclass and hasattr(metaclass, 'method') and metaclass.method or 'post'
        self.horizontal = True
        self.id = self.__class__.__name__.lower()
        self.inline = kwargs.pop('inline', False)
        self.partial = kwargs.pop('partial', False)
        self.perm_or_group = ()
        self.str_hidden = ''
        self.inner_forms = []
        self.configured_fieldsets = []
        self.submit_label = DEFAULT_SUBMIT_LABEL
        self.title = DEFAULT_FORM_TITLE
        self.is_inner = False
        self.captcha = False

        if self.method.lower() == 'post':
            kwargs['data'] = request.POST or None
            kwargs['files'] = request.FILES or None
        else:
            kwargs['data'] = request.GET or None

        if request.GET.get('popup'):
            prefix = kwargs.get('prefix', '')
            prefix = 'popup{}'.format(prefix)
            kwargs.update(prefix=prefix)

        super(Form, self).__init__(*args, **kwargs)

        if hasattr(self, 'instance') and not self.fieldsets and not self.inline:
            self.fieldsets = copy.deepcopy(get_metadata(self._meta.model, 'fieldsets', ()))

        if metaclass:
            self.title = hasattr(metaclass, 'title') and metaclass.title or ''
            self.icon = hasattr(metaclass, 'icon') and metaclass.icon or ''
            self.note = hasattr(metaclass, 'note') and metaclass.note or ''
            self.is_inner = hasattr(metaclass, 'is_inner') and metaclass.is_inner or False
            self.horizontal = hasattr(metaclass, 'horizontal') and metaclass.horizontal or False
            self.perm_or_group = hasattr(metaclass, 'perm_or_group') and iterable(metaclass.perm_or_group) or self.perm_or_group
            self.captcha = hasattr(metaclass, 'captcha') and metaclass.captcha or False

            if hasattr(metaclass, 'submit_label'):
                self.submit_label = metaclass.submit_label
            elif hasattr(self, 'instance'):
                self.submit_label = self.instance.pk and 'Atualizar' or 'Cadastrar'

            self.submit_style = hasattr(metaclass, 'submit_style') and metaclass.submit_style or 'default'
            self.method = hasattr(metaclass, 'method') and metaclass.method or 'post'

        for field_name in self.fields:
            if self.method.lower() == 'post' and field_name in self.request.GET:
                # field.widget = django_forms.HiddenInput()
                self.initial[field_name] = self.request.GET[field_name]

    def contextualize(self):

        for field_name in self.fields:
            field = self.fields[field_name]
            if hasattr(field, 'queryset'):
                if type(field) == django_forms.ModelMultipleChoiceField:
                    field.widget = MultipleModelChoiceField(field.queryset)
                if type(field) == CurrentUserField:
                    field.queryset = field.queryset.filter(pk=self.request.user.pk)
                    self.initial[field_name] = self.request.user.pk

                # if it is a model form
                if hasattr(self, 'instance'):
                    obj = None
                    role_username = get_metadata(field.queryset.model, 'role_username')
                    if role_username and self.request.user.groups.filter(name=field.queryset.model._meta.verbose_name):
                        obj = field.queryset.model.objects.get(**{role_username: self.request.user.username})
                    for subclass in field.queryset.model.__subclasses__():
                        role_username = get_metadata(subclass, 'role_username')
                        if role_username and self.request.user.groups.filter(name=subclass._meta.verbose_name):
                            obj = subclass.objects.get(**{role_username: self.request.user.username})
                    if obj:
                        groups = self.request.user.find_groups(self.perm_or_group, get_metadata(obj.__class__, 'verbose_name'))
                        # if the user is not superuser or there is only one group that allows the user to register the object
                        if not self.request.user.is_superuser and not groups.exists():
                            field.widget = widgets.HiddenInput(attrs={'value': obj.pk})
                            # field.widget = widgets.DisplayInput(obj)
                            continue

                if self.request.user.is_authenticated and (not hasattr(field, 'ignore_lookup') or not field.ignore_lookup):
                    if not self.is_inner:
                        field.queryset = field.queryset.all(self.request.user, obj=hasattr(self, 'instance') and self.instance or None)

                if True:#hasattr(field.queryset.model._meta, 'organization_lookup') or hasattr(field.queryset.model,'organization_ptr'):
                    from djangoplus.admin.models import Organization
                    if issubclass(field.queryset.model, Organization) and field.queryset.count() == 1:
                        if not isinstance(field, MultipleModelChoiceField):
                            obj = field.queryset[0]
                            field.widget = widgets.HiddenInput(attrs={'value': obj.pk})
                            # field.widget = widgets.DisplayInput(obj)

                if True:# hasattr(field.queryset.model._meta, 'unit_lookup') or hasattr(field.queryset.model, 'unit_ptr'):
                    from djangoplus.admin.models import Unit
                    if issubclass(field.queryset.model, Unit):
                        if not isinstance(field, MultipleModelChoiceField) and field.queryset.count() == 1:
                            obj = field.queryset[0]
                            field.widget = widgets.DisplayInput(obj)

                if hasattr(field.widget, 'lazy') and mobile(self.request):
                    field.widget.lazy = False

            if type(self.fields[field_name]) in [ModelChoiceField, MultipleModelChoiceField]:
                self.fields[field_name].widget.user = self.request.user

    def configure(self):

        from djangoplus.ui.components.forms import factory

        hidden_fields = []

        one_to_one_fields = dict()
        one_to_many_fields = dict()
        for name in list(self.fields.keys()):
            field = self.fields[name]
            if type(field) == OneToOneField:
                one_to_one_fields[name] = field
                del (self.fields[name])
            elif type(field) == OneToManyField:
                one_to_many_fields[name] = field
                del (self.fields[name])

        if not self.fieldsets:
            fields = list(self.fields.keys()) + list(one_to_one_fields.keys()) + list(one_to_many_fields.keys())
            if self.inline:
                self.fieldsets = (('', {'fields': (fields, )}),)
            else:
                self.fieldsets = (('', {'fields': fields}),)

        fieldset_field_names = []
        extra_fieldset_field_names = []
        for title, fieldset in self.fieldsets:
            field_names = fieldset.get('fields', ())
            relation_names = fieldset.get('relations', ())
            for name_or_tuple in tuple(field_names) + tuple(relation_names):
                for name in iterable(name_or_tuple):
                    fieldset_field_names.append(name)
        for field_name in list(self.fields.keys()):
            if field_name not in fieldset_field_names:
                extra_fieldset_field_names.append(field_name)
        if extra_fieldset_field_names:
            self.fieldsets += ('Outros', {'fields': extra_fieldset_field_names, }),

        for title, fieldset in self.fieldsets:
            title = '::' in title and title.split('::')[1] or title.split('::')[0]
            field_names = fieldset.get('fields', ())
            relation_names = fieldset.get('relations', ())

            configured_fieldset = dict(title=title, tuples=[], one_to_one=[], one_to_many=[])

            for name_or_tuple in tuple(field_names) + tuple(relation_names):
                fields = []
                for name in iterable(name_or_tuple):
                    if name in self.fields:
                        field = self.fields[name]
                        bf = BoundField(self, field, name)
                        if bf.is_hidden:
                            hidden_fields.append(bf)
                        else:
                            if bf.label:
                                label = conditional_escape(str(bf.label))
                                if self.label_suffix:
                                    if label[-1] not in ':?.!':
                                        label += self.label_suffix
                                label = label or ''
                            else:
                                label = ''

                            help_text = field.help_text or ''
                            label = str(label)[0:-1]
                            label = field.required and '{}<span class="text-danger">*</span>'.format(label) or label

                            d = dict(name=name, request=self.request, label=label, widget=bf,
                                     help_text=help_text)
                            fields.append(d)

                    elif name in one_to_one_fields:
                        field = one_to_one_fields[name]
                        one_to_one_id = getattr(self.instance, '{}_id'.format(name))
                        form = factory.get_one_to_one_form(self.request, self.instance, name, one_to_one_id,
                                                           partial=True, prefix=name)
                        required = field.required or form.data.get(form.prefix, None)
                        save = form.data.get(form.prefix, None)
                        if not required:
                            for field_name in form.fields:
                                form.fields[field_name].required = False
                        configured_fieldset['one_to_one'].append((field, form, required, save))
                        self.inner_forms.append(form)
                    elif name in one_to_many_fields:
                        field = one_to_many_fields[name]
                        one_to_many_forms = []

                        if self.instance.pk:
                            qs = getattr(self.instance, name).all()
                        else:
                            qs = field.queryset.filter(pk=0)
                        count = qs.count()
                        for i in range(0, field.one_to_many_max):
                            instance = i < count and qs[i] or None
                            form = factory.get_one_to_many_form(self.request, self.instance, name, partial=True,
                                                                inline=True, prefix='{}{}'.format(name, i),
                                                                instance=instance)
                            form.id = '{}-{}'.format(name, i)
                            form.hidden = i > count or field.one_to_many_count
                            required = form.data.get(form.prefix, None)
                            if not required:
                                for field_name in form.fields:
                                    form.fields[field_name].required = False
                            one_to_many_forms.append(form)
                            self.inner_forms.append(form)
                        one_to_many_count = None
                        if field.one_to_many_count:
                            if type(field.one_to_many_count) is int:
                                one_to_many_count = field.one_to_many_count
                            else:
                                app_label = get_metadata(qs.model, 'app_label')
                                if '__' in field.one_to_many_count:
                                    tokens = field.one_to_many_count.split('__')
                                    model_name = self.fields[tokens[0]].queryset.model.__name__.lower()
                                    model_lookup = '__'.join(tokens[1:])
                                    one_to_many_count = '{}:/view/{}/{}/PK/?one_to_many_count={}'.format(
                                        tokens[0], app_label, model_name, model_lookup)
                                else:
                                    one_to_many_count = field.one_to_many_count
                        configured_fieldset['one_to_many'].append((name, field, one_to_many_forms, one_to_many_count))

                if len(fields) > 2 or mobile(self.request):
                    self.horizontal = False

                configured_fieldset['tuples'].append(fields)

            self.configured_fieldsets.append(configured_fieldset)
        self.str_hidden = ''.join([str(x) for x in hidden_fields])

    def clean(self, *args, **kwargs):
        if self.request.POST and 'g-recaptcha-response' in self.request.POST:
            captcha_response = self.request.POST.get('g-recaptcha-response')
            captcha_url = 'https://www.google.com/recaptcha/api/siteverify'
            captcha_secret = settings.CAPTCHA_SECRET
            if captcha_response:
                data = dict(secret=captcha_secret, response=captcha_response)
                response = json.loads(urllib.request.urlopen(captcha_url, urllib.parse.urlencode(data).encode('utf-8')).read().decode('utf-8'))
                if not response.get('success'):
                    raise ValidationError('Confirme que você não é um robô.')
            else:
                raise ValidationError('Confirme que você não é um robô.')
        return super(Form, self).clean(*args, **kwargs)

    def is_valid(self, *args, **kwargs):
        self.contextualize()
        self.configure()
        valid = super(Form, self).is_valid(*args, **kwargs)
        for form in self.inner_forms:
            valid = form.is_valid(*args, **kwargs) and valid or False
        return valid

    def has_errors(self):
        if self.errors:
            return True
        for form in self.inner_forms:
            if form.errors:
                return True
        return False

    def __str__(self):
        if self.inline:
            for field_name in self.fields:
                self.fields[field_name].widget.attrs['placeholder'] = self.fields[field_name].label
                self.fields[field_name].widget.attrs['data-placeholder'] = self.fields[field_name].label
        return loader.render_to_string('form.html', {'self': self}, request=self.request)


class ModelForm(Form, django_forms.ModelForm):

    def save(self, *args, **kwargs):
        setattr(self.instance, 'request', self.request)
        for model_field in get_metadata(self.instance, 'fields'):
            # Se para o campo em questão foi definido um valor default. Ex: BooleanField(default=True)
            if model_field.default != NOT_PROVIDED:
                value = model_field.default
                if callable(value):
                    value = value()
                # Se não há nenhum valor no atributo da instância em questão, então será adotado
                # o valor default definido pelo desenvolvedor.
                if getattr(self.instance, model_field.name) is None:
                    setattr(self.instance, model_field.name, value)

        kwargs.update(commit=False)

        if len(self.inner_forms):
            with transaction.atomic():
                sid = transaction.savepoint()
                instance = super(ModelForm, self).save(*args, **kwargs)
                instance._post_save_form = self
                try:
                    instance.save()
                    sid and transaction.savepoint_commit(sid)
                except ValidationError as e:
                    sid and transaction.savepoint_rollback(sid)
                    raise e
        else:
            instance = super(ModelForm, self).save(*args, **kwargs)
            instance._post_save_form = self
            instance.save()

        return instance

    def save_121_and_12m(self):
        for fieldset in self.configured_fieldsets:
            for field, form, required, save in fieldset.get('one_to_one', ()):
                if save:
                    form.save()
                elif form.instance.pk:
                    form.instance.delete()

            for name, field, one_to_many_forms, one_to_many_count in fieldset.get('one_to_many', ()):
                for form in one_to_many_forms:
                    if form.data.get(form.prefix, None):
                        form.save()
                    else:
                        if form.instance.pk:
                            form.instance.delete()

class ModelFormOptions(object):
    def __init__(self, options=None):
        self.model = getattr(options, 'model', None)
        self.fields = getattr(options, 'fields', ())
        self.exclude = getattr(options, 'exclude', None)
        self.widgets = getattr(options, 'widgets', None)
        self.localized_fields = getattr(options, 'localized_fields', None)
        self.labels = getattr(options, 'labels', None)
        self.help_texts = getattr(options, 'help_texts', None)
        self.error_messages = getattr(options, 'error_messages', None)



import os
from subprocess import check_call
from setuptools import find_packages, setup
from setuptools.command.install import install

# pip install djangoplus[system-tools]

with open(os.path.join(os.path.dirname(__file__), 'djangoplus/README.md')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


class PostInstallCommand(install):

    def run(self):
        check_call("/bin/bash djangoplus/bin/setup-path".split())
        install.run(self)


setup(
    name='djangoplus',
    version='0.0.65',
    packages=find_packages(),
    install_requires=[
        'Django==2.0.3', 'xhtml2pdf==0.2.1', 'python-dateutil==2.7.1', 'selenium==3.11.0', 'xlwt==1.3.0',
        'xlrd==1.1.0', 'unicodecsv==0.14.1', 'qrcode==6.0'
    ],
    extras_require={
        'deploy': ['Pillow==5.0.2','cryptography==2.2.1', 'Fabric3==1.14.post1', 'dropbox==8.7.1']
    },
    scripts=[
        'djangoplus/bin/startproject', 'djangoplus/bin/runserver', 'djangoplus/bin/sync',
        'djangoplus/bin/install-tools', 'djangoplus/bin/setup-path'
    ],
    include_package_data=True,
    license='BSD License',
    description='Metadata-based web framework for the development of management information systems',
    long_description='',  # README,
    cmdclass={
        'install': PostInstallCommand,
    },
    url='http://djangoplus.net/',
    author='Breno Silva',
    author_email='brenokcc@yahoo.com.br',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# -*- coding: utf-8 -*-
import os
import warnings
from djangoplus.utils.webdriver import Browser
import json
from django.conf import settings
from djangoplus.test import cache
from django.core.management import call_command
from django.utils.translation import ugettext_lazy as _
from django.contrib.staticfiles.testing import LiveServerTestCase
from django.contrib.staticfiles.handlers import StaticFilesHandler


class TestStaticFilesHandler(StaticFilesHandler):
    def _middleware_chain(self, request):
        from django.http import HttpResponse
        return HttpResponse()


# StaticLiveServerTestCase
class TestCase(LiveServerTestCase):

    static_handler = TestStaticFilesHandler

    def __init__(self, *args, **kwargs):
        super(TestCase, self).__init__(*args, **kwargs)
        self.login_count = 0
        self.current_username = None
        self.current_password = None
        self.restored = False
        warnings.filterwarnings('ignore')

    def setUp(self):
        super(TestCase, self).setUp()
        self.slowly = False
        self.username = None
        self.browser = Browser(self.live_server_url)

    def create_superuser(self, username, password):
        from djangoplus.admin.models import User
        if not User.objects.filter(username=username).exists():
            User.objects.create_superuser(username, None, password)
        self.wait(1)

    def wait(self, seconds=1):
        self.browser.wait(seconds)

    def open(self, url):
        self.browser.open(url)

    def enter(self, name, value, submit=False, count=2):
        self.browser.enter(name, value, submit, count)

    def choose(self, name, value, count=2):
        self.browser.choose(name, value, count)

    def dont_see_error_message(self):
        self.browser.dont_see_error_message(self)

    def look_for(self, text, count=2):
        self.browser.look_for(text, count)

    def look_at_popup_window(self, count=2):
        self.browser.look_at_popup_window(count)

    def look_at(self, text, count=2):
        self.browser.look_at(text, count)

    def look_at_panel(self, text, count=2):
        self.browser.look_at_panel(text, count)

    def click_menu(self, *texts):
        self.browser.click_menu(*texts)

    def click_link(self, text):
        self.browser.click_link(text)

    def click_button(self, text):
        self.browser.click_button(text)

    def click_tab(self, text):
        self.browser.click_tab(text)

    def click_icon(self, name):
        self.browser.click_icon(name)

    def login(self, username, password):
        print('Logging as', username)
        self.login_count += 1
        self.current_username = username
        self.current_password = password
        if self.login_count >= cache.LOGIN_COUNT:
            self.dump()
            self.open('/admin/login/')
            self.enter(_('Username'), username)
            self.enter(_('Password'), password, True)
            self.wait()
            return True
        else:
            if not self.restored:
                self.restore()
            return False

    def logout(self):
        self.browser.logout()
        self.username = None

    def tearDown(self):
        super(TestCase, self).tearDown()
        self.browser.close()
        self.browser.service.stop()
        # len(self._resultForDoCleanups.errors)>0

    def dump(self, failed=False):
        file_path = '/tmp/{}.test'.format(settings.PROJECT_NAME)
        dump_file_path = '/tmp/{}.json'.format(settings.PROJECT_NAME)
        data = dict(login_count=self.login_count, username=self.current_username, password=self.current_password)
        open(file_path, 'w').write(json.dumps(data))
        output = open(dump_file_path,'w')
        app_labels = []
        for app in settings.INSTALLED_APPS:
            app_label = app.split('.')[-1]
            if app_label not in 'auth':
                app_labels.append(app_label)
        call_command('dumpdata', *app_labels, format='json', indent=3, stdout=output, skip_checks=True, verbosity=0)
        output.close()

    def restore(self):
        dump_file_path = '/tmp/{}.json'.format(settings.PROJECT_NAME)
        call_command('loaddata', dump_file_path)
        self.restored = True




# -*- coding: utf-8 -*-

import os
from djangoplus.conf import base_settings
from django.conf import settings
from django.core import serializers
from django.utils import termcolors
from django.core.management import call_command
from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand


def print_and_call(command, *args, **kwargs):
    kwargs.setdefault('interactive', True)
    print(termcolors.make_style(fg='cyan', opts=('bold',))('>>> {} {}{}'.format(
        command, ' '.join(args), ' '.join(['{}={}'.format(k, v) for k, v in list(kwargs.items())]))))
    call_command(command, *args, **kwargs)


class Command(BaseCommand):
    def handle(self, *args, **options):

        app_labels = []
        for app_label in settings.INSTALLED_APPS:
            if app_label not in base_settings.INSTALLED_APPS and '.' not in app_label:
                app_labels.append(app_label)


        print_and_call('makemigrations', *app_labels)
        print_and_call('migrate')

        # if there's no user, lets create admin user
        User = get_user_model()
        if not User.objects.exists():
            user = User.objects.create_superuser(settings.DEFAULT_SUPERUSER, None, settings.DEFAULT_PASSWORD)
            user.name = 'Admin'
            user.save()

        # if it is the production enverinoment, lets collect static files
        if os.path.exists(os.path.join(settings.BASE_DIR, 'logs')):
            print_and_call('collectstatic', clear=True, verbosity=0, interactive=False)



# -*- coding: utf-8 -*-
from djangoplus.ui import Component
from djangoplus.utils import terminal
from django.core.management.base import BaseCommand
from djangoplus.docs.utils import extract_documentation


class Command(BaseCommand):

    def handle(self, *args, **options):
        for cls in Component.__subclasses__():
            name = terminal.bold(cls.__name__)
            description = extract_documentation(cls)
            print('{}: {}'.format(name, description))


# -*- coding: utf-8 -*-
import base64
from django.conf import settings

try:
    from cryptography.fernet import Fernet
    cipher = Fernet(base64.urlsafe_b64encode(settings.SECRET_KEY.encode()[0:32]).decode())
except ImportError:
    cipher = None


def encrypt(text, b16=False):
    if cipher:
        s = cipher.encrypt(str(text).encode('utf-8'))
    else:
        s = base64.b64encode(text.encode('utf-8'))
    if b16:
        s = base64.b16encode(s)
    return s.decode('utf-8')


def decrypt(text, b16=False):
    if b16:
        text = base64.b16decode(text.encode('utf-8'))
    else:
        text = text.encode('utf-8')
    if cipher:
        s = cipher.decrypt(text)
    else:
        s = base64.b64decode(text.decode('utf-8'))
    return s.decode('utf-8')
